@@ -26,9 +26,9 @@
 #ifndef DV_TYPES_H
 #define DV_TYPES_H
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif // HAVE_CONFIG_H
+#if HAVE_CONFIG_H
+# include <config.h>
+#endif
 
 #if HAVE_LIBPOPT
 #include <popt.h>
@@ -77,7 +77,7 @@
 #define __FUNCTION__ __FILE__ // Less specific info, but it's a string.
 #endif
 
-#define DV_AUDIO_MAX_SAMPLES 1920
+#define DV_AUDIO_MAX_SAMPLES 1944
 
 #define DV_AUDIO_OPT_FREQUENCY    0
 #define DV_AUDIO_OPT_QUANTIZATION 1
@@ -485,6 +485,7 @@ typedef struct {
   int    clamp_luma;
   int    clamp_chroma;
   int    frame_count;
+  int    samples_this_frame;
   /* the below are private */
   short *img_y;
   short *img_cr;
