@@ -23,6 +23,10 @@
  *  The libdv homepage is http://libdv.sourceforge.net/.  
  */
  
+#if HAVE_CONFIG_H
+# include <config.h>
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <signal.h>
@@ -56,7 +60,7 @@
 // #define ARCH_X86 0
 
 #if !ARCH_X86
-inline gint f2b(float f)
+static inline int f2b(float f)
 {
 	int b = rint(f);
 	if (b < 0)
@@ -68,7 +72,7 @@ inline gint f2b(float f)
 }
 
 
-inline gint f2sb(float f)
+static inline int f2sb(float f)
 {
 	int b = rint(f);
 	
@@ -76,7 +80,7 @@ inline gint f2sb(float f)
 }
 #endif
 
-extern void rgbtoycb_mmx(unsigned char* inPtr, int rows, int columns,
+extern void _dv_rgbtoycb_mmx(unsigned char* inPtr, int rows, int columns,
 			 short* outyPtr, short* outuPtr, short* outvPtr);
 
 void dv_enc_rgb_to_ycb(unsigned char* img_rgb, int height,
@@ -157,7 +161,7 @@ void dv_enc_rgb_to_ycb(unsigned char* img_rgb, int height,
 	}
 #endif
 #else
-	rgbtoycb_mmx(img_rgb, height, DV_WIDTH, (short*) img_y,
+	_dv_rgbtoycb_mmx(img_rgb, height, DV_WIDTH, (short*) img_y,
 		     (short*) img_cr, (short*) img_cb);
 	emms();
 #endif
@@ -206,12 +210,12 @@ static int need_dct_248_transposed(dv_coeff_t * bl)
 
 #else
 
-extern int need_dct_248_mmx_rows(dv_coeff_t * bl);
+extern int _dv_need_dct_248_mmx_rows(dv_coeff_t * bl);
 
-extern void transpose_mmx(short * dst);
-extern void ppm_copy_y_block_mmx(short * dst, short * src);
-extern void ppm_copy_pal_c_block_mmx(short * dst, short * src);
-extern void ppm_copy_ntsc_c_block_mmx(short * dst, short * src);
+extern void _dv_transpose_mmx(short * dst);
+extern void _dv_ppm_copy_y_block_mmx(short * dst, short * src);
+extern void _dv_ppm_copy_pal_c_block_mmx(short * dst, short * src);
+extern void _dv_ppm_copy_ntsc_c_block_mmx(short * dst, short * src);
 
 static void finish_mb_mmx(dv_macroblock_t* mb)
 {
@@ -226,21 +230,21 @@ static void finish_mb_mmx(dv_macroblock_t* mb)
 	} else {
 		for (b = 0; b < 6; b++) {
 			need_dct_248_rows[b]
-				= need_dct_248_mmx_rows(bl[b].coeffs) + 1;
+				= _dv_need_dct_248_mmx_rows(bl[b].coeffs) + 1;
 		}
 	}
-	transpose_mmx(bl[0].coeffs);
-	transpose_mmx(bl[1].coeffs);
-	transpose_mmx(bl[2].coeffs);
-	transpose_mmx(bl[3].coeffs);
-	transpose_mmx(bl[4].coeffs);
-	transpose_mmx(bl[5].coeffs);
+	_dv_transpose_mmx(bl[0].coeffs);
+	_dv_transpose_mmx(bl[1].coeffs);
+	_dv_transpose_mmx(bl[2].coeffs);
+	_dv_transpose_mmx(bl[3].coeffs);
+	_dv_transpose_mmx(bl[4].coeffs);
+	_dv_transpose_mmx(bl[5].coeffs);
 
 	if (force_dct == -1) {
 		for (b = 0; b < 6; b++) {
 			bl[b].dct_mode = 
 				((need_dct_248_rows[b] * 65536 / 
-				  (need_dct_248_mmx_rows(bl[b].coeffs) + 1))
+				  (_dv_need_dct_248_mmx_rows(bl[b].coeffs) + 1))
 				 > DCT_248_THRESHOLD) ? DV_DCT_248 : DV_DCT_88;
 		}
 	}
@@ -357,7 +361,7 @@ static void ppm_fill_macroblock(dv_macroblock_t *mb, int isPAL)
 	dv_block_t* bl = mb->b;
 
 #if !ARCH_X86
-	if (isPAL || mb->x == DV_WIDTH- 16) { /* PAL or rightmost NTSC block */
+	if (isPAL) { /* PAL */
 		int i,j;
 		for (j = 0; j < 8; j++) {
 			for (i = 0; i < 8; i++) {
@@ -382,6 +386,43 @@ static void ppm_fill_macroblock(dv_macroblock_t *mb, int isPAL)
 						+ x / 2 + i]) >> 1;
 			}
 		}
+	} else if (x == DV_WIDTH- 16) { /* rightmost NTSC block */
+		int i,j;
+		for (j = 0; j < 8; j++) {
+			for (i = 0; i < 8; i++) {
+				bl[0].coeffs[8 * i + j] = 
+					img_y[(y + j) * DV_WIDTH +  x + i];
+				bl[1].coeffs[8 * i + j] = 
+					img_y[(y + j) * DV_WIDTH +  x + 8 + i];
+				bl[2].coeffs[8 * i + j] = 
+					img_y[(y + 8 + j) * DV_WIDTH + x + i];
+				bl[3].coeffs[8 * i + j] = 
+					img_y[(y + 8 + j) * DV_WIDTH 
+					     + x + 8 + i];
+			}
+			for (i = 0; i < 4; i++) {
+				bl[4].coeffs[8 * i + j] = 
+					(img_cr[(y + j) * DV_WIDTH/2
+					       + x / 2 + i*2]
+					 + img_cr[(y + j) * DV_WIDTH/2 
+						 + x / 2 + 1 + i*2]) >> 1;
+				bl[5].coeffs[8 * i + j] = 
+					(img_cb[(y + j) * DV_WIDTH/2
+					       + x / 2 + i*2]
+					 + img_cb[(y + j) * DV_WIDTH/2 
+						 + x / 2 + 1 + i*2]) >> 1;
+				bl[4].coeffs[8 * (i + 4) + j] = 
+					(img_cr[(y + j + 8) * DV_WIDTH/2
+					       + x / 2 + i*2]
+					 + img_cr[(y + j + 8) * DV_WIDTH/2 
+						 + x / 2 + 1 + i*2]) >> 1;
+				bl[5].coeffs[8 * (i + 4) + j] = 
+					(img_cb[(y + j + 8) * DV_WIDTH/2
+					       + x / 2 + i*2]
+					 + img_cb[(y + j + 8) * DV_WIDTH/2 
+						 + x / 2 + 1 + i*2]) >> 1;
+			}
+		}
 	} else {                        /* NTSC */
 		int i,j;
 		for (j = 0; j < 8; j++) {
@@ -420,25 +461,59 @@ static void ppm_fill_macroblock(dv_macroblock_t *mb, int isPAL)
 		}
 	}
 #else
-	if (isPAL || mb->x == DV_WIDTH- 16) { /* PAL or rightmost NTSC block */
+	if (isPAL) { /* PAL */
 		short* start_y = img_y + y * DV_WIDTH + x;
-		ppm_copy_y_block_mmx(bl[0].coeffs, start_y);
-		ppm_copy_y_block_mmx(bl[1].coeffs, start_y + 8);
-		ppm_copy_y_block_mmx(bl[2].coeffs, start_y + 8 * DV_WIDTH);
-		ppm_copy_y_block_mmx(bl[3].coeffs, start_y + 8 * DV_WIDTH + 8);
-		ppm_copy_pal_c_block_mmx(bl[4].coeffs,
+		_dv_ppm_copy_y_block_mmx(bl[0].coeffs, start_y);
+		_dv_ppm_copy_y_block_mmx(bl[1].coeffs, start_y + 8);
+		_dv_ppm_copy_y_block_mmx(bl[2].coeffs, start_y + 8 * DV_WIDTH);
+		_dv_ppm_copy_y_block_mmx(bl[3].coeffs, start_y + 8 * DV_WIDTH + 8);
+		_dv_ppm_copy_pal_c_block_mmx(bl[4].coeffs,
 					 img_cr+y * DV_WIDTH/2+ x/2);
-		ppm_copy_pal_c_block_mmx(bl[5].coeffs,
+		_dv_ppm_copy_pal_c_block_mmx(bl[5].coeffs,
 					 img_cb+y * DV_WIDTH/2+ x/2);
-	} else {                              /* NTSC */
+	} else if (mb->x == DV_WIDTH- 16) { /* rightmost NTSC block */
 		short* start_y = img_y + y * DV_WIDTH + x;
-		ppm_copy_y_block_mmx(bl[0].coeffs, start_y);
-		ppm_copy_y_block_mmx(bl[1].coeffs, start_y + 8);
-		ppm_copy_y_block_mmx(bl[2].coeffs, start_y + 16);
-		ppm_copy_y_block_mmx(bl[3].coeffs, start_y + 24);
-		ppm_copy_ntsc_c_block_mmx(bl[4].coeffs,
+		int i,j;
+
+		_dv_ppm_copy_y_block_mmx(bl[0].coeffs, start_y);
+		_dv_ppm_copy_y_block_mmx(bl[1].coeffs, start_y + 8);
+		_dv_ppm_copy_y_block_mmx(bl[2].coeffs, start_y + 8 * DV_WIDTH);
+		_dv_ppm_copy_y_block_mmx(bl[3].coeffs, start_y + 8 * DV_WIDTH + 8);
+
+		for (j = 0; j < 8; j++) {
+			for (i = 0; i < 4; i++) {
+				bl[4].coeffs[8 * j + i] = 
+					(img_cr[(y + j) * DV_WIDTH/2
+					       + x / 2 + i*2]
+					 + img_cr[(y + j) * DV_WIDTH/2 
+						 + x / 2 + 1 + i*2]) >> 1;
+				bl[5].coeffs[8 * j + i] = 
+					(img_cb[(y + j) * DV_WIDTH/2
+					       + x / 2 + i*2]
+					 + img_cb[(y + j) * DV_WIDTH/2 
+						 + x / 2 + 1 + i*2]) >> 1;
+				bl[4].coeffs[8 * j + i + 4] = 
+					(img_cr[(y + j + 8) * DV_WIDTH/2
+					       + x / 2 + i*2]
+					 + img_cr[(y + j + 8) * DV_WIDTH/2 
+						 + x / 2 + 1 + i*2]) >> 1;
+				bl[5].coeffs[8 * j + i + 4] = 
+					(img_cb[(y + j + 8) * DV_WIDTH/2
+					       + x / 2 + i*2]
+					 + img_cb[(y + j + 8) * DV_WIDTH/2 
+						 + x / 2 + 1 + i*2]) >> 1;
+			}
+		}
+
+	} else { /* NTSC */
+		short* start_y = img_y + y * DV_WIDTH + x;
+		_dv_ppm_copy_y_block_mmx(bl[0].coeffs, start_y);
+		_dv_ppm_copy_y_block_mmx(bl[1].coeffs, start_y + 8);
+		_dv_ppm_copy_y_block_mmx(bl[2].coeffs, start_y + 16);
+		_dv_ppm_copy_y_block_mmx(bl[3].coeffs, start_y + 24);
+		_dv_ppm_copy_ntsc_c_block_mmx(bl[4].coeffs,
 					  img_cr + y*DV_WIDTH/2 + x/2);
-		ppm_copy_ntsc_c_block_mmx(bl[5].coeffs,
+		_dv_ppm_copy_ntsc_c_block_mmx(bl[5].coeffs,
 					  img_cb + y*DV_WIDTH/2 + x/2);
 	}
 
@@ -544,6 +619,24 @@ static int pgm_skip(const char* filename, int * isPAL)
 	return rval;
 }
 
+static inline short pgm_get_cr_ntsc(int y, int x)
+{
+	return ((((short) real_readbuf[DV_NTSC_HEIGHT * DV_WIDTH+ DV_WIDTH/2
+				      + y * DV_WIDTH + x]) - 128)
+		+ (((short) real_readbuf[DV_NTSC_HEIGHT * DV_WIDTH+DV_WIDTH/2
+					+ y * DV_WIDTH + x + 1]) - 128))
+						<< (DCT_YUV_PRECISION - 1);
+}
+
+static inline short pgm_get_cb_ntsc(int y, int x)
+{
+	return ((((short) real_readbuf[DV_NTSC_HEIGHT * DV_WIDTH
+				      + y * DV_WIDTH + x]) - 128)
+		+ (((short) real_readbuf[DV_NTSC_HEIGHT * DV_WIDTH
+					+ y * DV_WIDTH + x + 1]) - 128))
+						<< (DCT_YUV_PRECISION - 1);
+}
+
 #if !ARCH_X86
 static inline short pgm_get_y(int y, int x)
 {
@@ -566,28 +659,10 @@ static inline short pgm_get_cb_pal(int y, int x)
 
 }
 
-static inline short pgm_get_cr_ntsc(int y, int x)
-{
-	return ((((short) real_readbuf[DV_NTSC_HEIGHT * DV_WIDTH+ DV_WIDTH/2
-				      + y * DV_WIDTH + x]) - 128)
-		+ (((short) real_readbuf[DV_NTSC_HEIGHT * DV_WIDTH+DV_WIDTH/2
-					+ y * DV_WIDTH + x + 1]) - 128))
-						<< (DCT_YUV_PRECISION - 1);
-}
-
-static inline short pgm_get_cb_ntsc(int y, int x)
-{
-	return ((((short) real_readbuf[DV_NTSC_HEIGHT * DV_WIDTH
-				      + y * DV_WIDTH + x]) - 128)
-		+ (((short) real_readbuf[DV_NTSC_HEIGHT * DV_WIDTH
-					+ y * DV_WIDTH + x + 1]) - 128))
-						<< (DCT_YUV_PRECISION - 1);
-}
-
 #else
-extern void pgm_copy_y_block_mmx(short * dst, unsigned char * src);
-extern void pgm_copy_pal_c_block_mmx(short * dst, unsigned char * src);
-extern void pgm_copy_ntsc_c_block_mmx(short * dst, unsigned char * src);
+extern void _dv_pgm_copy_y_block_mmx(short * dst, unsigned char * src);
+extern void _dv_pgm_copy_pal_c_block_mmx(short * dst, unsigned char * src);
+extern void _dv_pgm_copy_ntsc_c_block_mmx(short * dst, unsigned char * src);
 #endif
 
 static void pgm_fill_macroblock(dv_macroblock_t *mb, int isPAL)
@@ -596,7 +671,7 @@ static void pgm_fill_macroblock(dv_macroblock_t *mb, int isPAL)
 	int x = mb->x;
 	dv_block_t* bl = mb->b;
 #if !ARCH_X86
-	if (isPAL || mb->x == DV_WIDTH- 16) { /* PAL or rightmost NTSC block */
+	if (isPAL) { /* PAL */
 		int i,j;
 		for (j = 0; j < 8; j++) {
 			for (i = 0; i < 8; i++) {
@@ -610,22 +685,46 @@ static void pgm_fill_macroblock(dv_macroblock_t *mb, int isPAL)
 					pgm_get_cb_pal(y/2+j, x/2+i);
 			}
 		}
-	} else {                        /* NTSC */
+	} else if (x == DV_WIDTH- 16) { /* rightmost NTSC block */
 		int i,j;
-		for (i = 0; i < 8; i++) {
-			for (j = 0; j < 8; j++) {
+		for (j = 0; j < 8; j++) {
+			for (i = 0; i < 8; i++) {
+				bl[0].coeffs[8*i + j] = pgm_get_y(y+j,x+i);
+				bl[1].coeffs[8*i + j] = pgm_get_y(y+j,x+8+i);
+				bl[2].coeffs[8*i + j] = pgm_get_y(y+8+j,x+i);
+				bl[3].coeffs[8*i + j] = pgm_get_y(y+8+j,x+8+i);
+			}
+			for (i = 0; i < 4; i++) {
+				bl[4].coeffs[8*j + i*2] = 
+					bl[4].coeffs[8*j + i*2 + 1] = 
+					pgm_get_cr_ntsc(y/2 + j, x/2 + i * 2);
+				bl[5].coeffs[8*j + i*2] = 
+					bl[5].coeffs[8*j + i*2 + 1] = 
+					pgm_get_cb_ntsc(y/2 + j, x/2 + i * 2);
+				bl[4].coeffs[8*j + (i+4)*2] = 
+					bl[4].coeffs[8*j + (i+4)*2 + 1] = 
+					pgm_get_cr_ntsc(y/2 + j +8, x/2 + i * 2);
+				bl[5].coeffs[8*j + (i+4)*2] = 
+					bl[5].coeffs[8*j + (i+4)*2 + 1] = 
+					pgm_get_cb_ntsc(y/2 + j +8, x/2 + i * 2);
+			}
+		}
+	} else { /* NTSC */
+		int i,j;
+		for (j = 0; j < 8; j++) {
+			for (i = 0; i < 8; i++) {
 				bl[0].coeffs[8*i + j] = pgm_get_y(y+j,x+i);
 				bl[1].coeffs[8*i + j] = pgm_get_y(y+j,x+8+i);
 				bl[2].coeffs[8*i + j] = pgm_get_y(y+j,x+16+i);
 				bl[3].coeffs[8*i + j] = pgm_get_y(y+j,x+24+i);
 			}
-			for (j = 0; j < 4; j++) {
-				bl[4].coeffs[8*i + j*2] = 
-					bl[4].coeffs[8*i + j*2 + 1] = 
-					pgm_get_cr_ntsc(y + j, x/2 + i * 2);
-				bl[5].coeffs[8*i + j*2] = 
-					bl[5].coeffs[8*i + j*2 + 1] = 
-					pgm_get_cb_ntsc(y + j, x/2 + i * 2);
+			for (i = 0; i < 4; i++) {
+				bl[4].coeffs[8*j + i*2] = 
+					bl[4].coeffs[8*j + i*2 + 1] = 
+					pgm_get_cr_ntsc(y/2 + i, x/2 + j * 2);
+				bl[5].coeffs[8*j + i*2] = 
+					bl[5].coeffs[8*j + i*2 + 1] = 
+					pgm_get_cb_ntsc(y/2 + i, x/2 + j * 2);
 			}
 		}
 	}
@@ -642,36 +741,67 @@ static void pgm_fill_macroblock(dv_macroblock_t *mb, int isPAL)
 		}
 	}
 #else
-	if (isPAL || mb->x == DV_WIDTH- 16) { /* PAL or rightmost NTSC block */
+	if (isPAL) { /* PAL */
 		unsigned char* start_y = real_readbuf + y * DV_WIDTH + x;
+		unsigned char* img_cr = real_readbuf 
+			+ DV_WIDTH * DV_PAL_HEIGHT + DV_WIDTH / 2;
+		unsigned char* img_cb = real_readbuf 
+			+ DV_WIDTH * DV_PAL_HEIGHT;
+
+		_dv_pgm_copy_y_block_mmx(bl[0].coeffs, start_y);
+		_dv_pgm_copy_y_block_mmx(bl[1].coeffs, start_y + 8);
+		_dv_pgm_copy_y_block_mmx(bl[2].coeffs, start_y + 8 * DV_WIDTH);
+		_dv_pgm_copy_y_block_mmx(bl[3].coeffs, start_y + 8 * DV_WIDTH + 8);
+		_dv_pgm_copy_pal_c_block_mmx(bl[4].coeffs,
+					 img_cr + y * DV_WIDTH / 2 + x / 2);
+		_dv_pgm_copy_pal_c_block_mmx(bl[5].coeffs,
+					 img_cb + y * DV_WIDTH / 2 + x / 2);
+	} else if (x == DV_WIDTH- 16) {  /* rightmost NTSC block */
+		unsigned char* start_y = real_readbuf + y * DV_WIDTH + x;
+#if 0
 		unsigned char* img_cr = real_readbuf 
 			+ (isPAL ? (DV_WIDTH * DV_PAL_HEIGHT)
 			   : (DV_WIDTH * DV_NTSC_HEIGHT)) + DV_WIDTH / 2;
 		unsigned char* img_cb = real_readbuf 
 			+ (isPAL ? (DV_WIDTH * DV_PAL_HEIGHT) 
 			   : (DV_WIDTH * DV_NTSC_HEIGHT));
+#endif
+		int i,j;
 
-		pgm_copy_y_block_mmx(bl[0].coeffs, start_y);
-		pgm_copy_y_block_mmx(bl[1].coeffs, start_y + 8);
-		pgm_copy_y_block_mmx(bl[2].coeffs, start_y + 8 * DV_WIDTH);
-		pgm_copy_y_block_mmx(bl[3].coeffs, start_y + 8 * DV_WIDTH + 8);
-		pgm_copy_pal_c_block_mmx(bl[4].coeffs,
-					 img_cr + y * DV_WIDTH / 2 + x / 2);
-		pgm_copy_pal_c_block_mmx(bl[5].coeffs,
-					 img_cb + y * DV_WIDTH / 2 + x / 2);
+		_dv_pgm_copy_y_block_mmx(bl[0].coeffs, start_y);
+		_dv_pgm_copy_y_block_mmx(bl[1].coeffs, start_y + 8);
+		_dv_pgm_copy_y_block_mmx(bl[2].coeffs, start_y + 8 * DV_WIDTH);
+		_dv_pgm_copy_y_block_mmx(bl[3].coeffs, start_y + 8 * DV_WIDTH + 8);
+
+		for (j = 0; j < 8; j++) {
+			for (i = 0; i < 4; i++) {
+				bl[4].coeffs[8*j + i*2] = 
+					bl[4].coeffs[8*j + i*2 + 1] = 
+					pgm_get_cr_ntsc(y/2 + j, x/2 + i * 2);
+				bl[5].coeffs[8*j + i*2] = 
+					bl[5].coeffs[8*j + i*2 + 1] = 
+					pgm_get_cb_ntsc(y/2 + j, x/2 + i * 2);
+				bl[4].coeffs[8*j + (i+4)*2] = 
+					bl[4].coeffs[8*j + (i+4)*2 + 1] = 
+					pgm_get_cr_ntsc(y/2 + j +8, x/2 + i * 2);
+				bl[5].coeffs[8*j + (i+4)*2] = 
+					bl[5].coeffs[8*j + (i+4)*2 + 1] = 
+					pgm_get_cb_ntsc(y/2 + j +8, x/2 + i * 2);
+			}
+		}
 	} else {                              /* NTSC */
 		unsigned char* start_y = real_readbuf + y * DV_WIDTH + x;
 		unsigned char* img_cr = real_readbuf 
 			+ DV_WIDTH * DV_NTSC_HEIGHT + DV_WIDTH / 2;
 		unsigned char* img_cb = real_readbuf 
 			+ DV_WIDTH * DV_NTSC_HEIGHT;
-		pgm_copy_y_block_mmx(bl[0].coeffs, start_y);
-		pgm_copy_y_block_mmx(bl[1].coeffs, start_y + 8);
-		pgm_copy_y_block_mmx(bl[2].coeffs, start_y + 16);
-		pgm_copy_y_block_mmx(bl[3].coeffs, start_y + 24);
-		pgm_copy_ntsc_c_block_mmx(bl[4].coeffs,
+		_dv_pgm_copy_y_block_mmx(bl[0].coeffs, start_y);
+		_dv_pgm_copy_y_block_mmx(bl[1].coeffs, start_y + 8);
+		_dv_pgm_copy_y_block_mmx(bl[2].coeffs, start_y + 16);
+		_dv_pgm_copy_y_block_mmx(bl[3].coeffs, start_y + 24);
+		_dv_pgm_copy_ntsc_c_block_mmx(bl[4].coeffs,
 					  img_cr + y * DV_WIDTH / 2 + x / 2);
-		pgm_copy_ntsc_c_block_mmx(bl[5].coeffs,
+		_dv_pgm_copy_ntsc_c_block_mmx(bl[5].coeffs,
 					  img_cb + y * DV_WIDTH / 2 + x / 2);
 	}
 
@@ -778,6 +908,24 @@ static int video_load(const char* filename, int * isPAL)
 	return 0;
 }
 
+static inline short video_get_cr_ntsc(int y, int x)
+{
+	return ((((short) real_readbuf[DV_NTSC_HEIGHT * DV_WIDTH
+				      + y * DV_WIDTH + x]) - 128)
+		+ (((short) real_readbuf[DV_NTSC_HEIGHT * DV_WIDTH
+					+ y * DV_WIDTH + x + 1]) - 128))
+						<< (DCT_YUV_PRECISION - 1);
+}
+
+static inline short video_get_cb_ntsc(int y, int x)
+{
+	return ((((short) real_readbuf[DV_NTSC_HEIGHT * DV_WIDTH*3/2
+				      + y * DV_WIDTH + x]) - 128)
+		+ (((short) real_readbuf[DV_NTSC_HEIGHT * DV_WIDTH*3/2
+					+ y * DV_WIDTH + x + 1]) - 128))
+						<< (DCT_YUV_PRECISION - 1);
+}
+
 #if !ARCH_X86
 static inline short video_get_y(int y, int x)
 {
@@ -803,29 +951,11 @@ static inline short video_get_cb_pal(int y, int x)
 		   - 128)) << (DCT_YUV_PRECISION - 1);
 }
 
-static inline short video_get_cr_ntsc(int y, int x)
-{
-	return ((((short) real_readbuf[DV_NTSC_HEIGHT * DV_WIDTH
-				      + y * DV_WIDTH + x]) - 128)
-		+ (((short) real_readbuf[DV_NTSC_HEIGHT * DV_WIDTH
-					+ y * DV_WIDTH + x + 1]) - 128))
-						<< (DCT_YUV_PRECISION - 1);
-}
-
-static inline short video_get_cb_ntsc(int y, int x)
-{
-	return ((((short) real_readbuf[DV_NTSC_HEIGHT * DV_WIDTH*3/2
-				      + y * DV_WIDTH + x]) - 128)
-		+ (((short) real_readbuf[DV_NTSC_HEIGHT * DV_WIDTH*3/2
-					+ y * DV_WIDTH + x + 1]) - 128))
-						<< (DCT_YUV_PRECISION - 1);
-}
-
 
 #else
-extern void video_copy_y_block_mmx(short * dst, unsigned char * src);
-extern void video_copy_pal_c_block_mmx(short * dst, unsigned char * src);
-extern void video_copy_ntsc_c_block_mmx(short * dst, unsigned char * src);
+extern void _dv_video_copy_y_block_mmx(short * dst, unsigned char * src);
+extern void _dv_video_copy_pal_c_block_mmx(short * dst, unsigned char * src);
+extern void _dv_video_copy_ntsc_c_block_mmx(short * dst, unsigned char * src);
 #endif
 
 
@@ -835,7 +965,7 @@ static void video_fill_macroblock(dv_macroblock_t *mb, int isPAL)
 	int x = mb->x;
 	dv_block_t* bl = mb->b;
 #if !ARCH_X86
-	if (isPAL || mb->x == DV_WIDTH- 16) { /* PAL or rightmost NTSC block */
+	if (isPAL) { /* PAL */
 		int i,j;
 		for (j = 0; j < 8; j++) {
 			for (i = 0; i < 8; i++) {
@@ -849,6 +979,27 @@ static void video_fill_macroblock(dv_macroblock_t *mb, int isPAL)
 					video_get_cb_pal(y/2+j, x/2+i);
 			}
 		}
+	} else if ( x == DV_WIDTH- 16) { /* rightmost NTSC block */
+		int i,j;
+		for (j = 0; j < 8; j++) {
+			for (i = 0; i < 8; i++) {
+				bl[0].coeffs[8*i + j] = video_get_y(y+j,x+i);
+				bl[1].coeffs[8*i + j] = video_get_y(y+j,x+8+i);
+				bl[2].coeffs[8*i + j] = video_get_y(y+8+j,x+i);
+				bl[3].coeffs[8*i + j] = video_get_y(y+8+j,x+8+i);
+			}
+			for (i = 0; i < 4; i++) {
+				bl[4].coeffs[8*i + j] = 
+					video_get_cr_ntsc(y/2+j, x/2+i);
+				bl[5].coeffs[8*i + j] = 
+					video_get_cb_ntsc(y/2+j, x/2+i);
+				bl[4].coeffs[8*(i+4) + j] = 
+					video_get_cr_ntsc(y/2+j+8, x/2+i);
+				bl[5].coeffs[8*(i+4) + j] = 
+					video_get_cb_ntsc(y/2+j+8, x/2+i);
+			}
+		}
+		
 	} else {                        /* NTSC */
 		int i,j;
 		for (i = 0; i < 8; i++) {
@@ -877,7 +1028,7 @@ static void video_fill_macroblock(dv_macroblock_t *mb, int isPAL)
 		}
 	}
 #else
-	if (isPAL || mb->x == DV_WIDTH- 16) { /* PAL or rightmost NTSC block */
+	if (isPAL) { /* PAL */
 		unsigned char* start_y = real_readbuf + y * DV_WIDTH + x;
 		unsigned char* img_cr = real_readbuf 
 			+ (isPAL ? DV_WIDTH * DV_PAL_HEIGHT * 3/2  
@@ -886,27 +1037,57 @@ static void video_fill_macroblock(dv_macroblock_t *mb, int isPAL)
 			+ (isPAL ? DV_WIDTH * DV_PAL_HEIGHT
 			   : DV_WIDTH * DV_NTSC_HEIGHT);
 
-		video_copy_y_block_mmx(bl[0].coeffs, start_y);
-		video_copy_y_block_mmx(bl[1].coeffs, start_y + 8);
-		video_copy_y_block_mmx(bl[2].coeffs, start_y + 8 * DV_WIDTH);
-		video_copy_y_block_mmx(bl[3].coeffs, start_y + 8 * DV_WIDTH+8);
-		video_copy_pal_c_block_mmx(bl[4].coeffs,
+		_dv_video_copy_y_block_mmx(bl[0].coeffs, start_y);
+		_dv_video_copy_y_block_mmx(bl[1].coeffs, start_y + 8);
+		_dv_video_copy_y_block_mmx(bl[2].coeffs, start_y + 8 * DV_WIDTH);
+		_dv_video_copy_y_block_mmx(bl[3].coeffs, start_y + 8 * DV_WIDTH+8);
+		_dv_video_copy_pal_c_block_mmx(bl[4].coeffs,
 					 img_cr + y * DV_WIDTH / 2 + x / 2);
-		video_copy_pal_c_block_mmx(bl[5].coeffs,
+		_dv_video_copy_pal_c_block_mmx(bl[5].coeffs,
 					 img_cb + y * DV_WIDTH / 2 + x / 2);
+	} else if (x == DV_WIDTH- 16) {       /* rightmost NTSC block */
+		unsigned char* start_y = real_readbuf + y * DV_WIDTH + x;
+#if 0
+		unsigned char* img_cr = real_readbuf 
+			+ (isPAL ? DV_WIDTH * DV_PAL_HEIGHT * 3/2  
+			   : DV_WIDTH * DV_NTSC_HEIGHT * 3/2);
+		unsigned char* img_cb = real_readbuf 
+			+ (isPAL ? DV_WIDTH * DV_PAL_HEIGHT
+			   : DV_WIDTH * DV_NTSC_HEIGHT);
+#endif
+		int i,j;
+
+		_dv_video_copy_y_block_mmx(bl[0].coeffs, start_y);
+		_dv_video_copy_y_block_mmx(bl[1].coeffs, start_y + 8);
+		_dv_video_copy_y_block_mmx(bl[2].coeffs, start_y + 8 * DV_WIDTH);
+		_dv_video_copy_y_block_mmx(bl[3].coeffs, start_y + 8 * DV_WIDTH+8);
+		
+
+		for (j = 0; j < 8; j++) {
+			for (i = 0; i < 4; i++) {
+				bl[4].coeffs[8*j + i] = 
+					video_get_cr_ntsc(y/2+j, x/2+i);
+				bl[5].coeffs[8*j + i] = 
+					video_get_cb_ntsc(y/2+j, x/2+i);
+				bl[4].coeffs[8*j + (i+4)] = 
+					video_get_cr_ntsc(y/2+j+8, x/2+i);
+				bl[5].coeffs[8*j + (i+4)] = 
+					video_get_cb_ntsc(y/2+j+8, x/2+i);
+			}
+		}
 	} else {                              /* NTSC */
 		unsigned char* start_y = real_readbuf + y * DV_WIDTH + x;
 		unsigned char* img_cr = real_readbuf 
 			+ DV_WIDTH * DV_NTSC_HEIGHT * 3 / 2;
 		unsigned char* img_cb = real_readbuf 
 			+ DV_WIDTH * DV_NTSC_HEIGHT;
-		video_copy_y_block_mmx(bl[0].coeffs, start_y);
-		video_copy_y_block_mmx(bl[1].coeffs, start_y + 8);
-		video_copy_y_block_mmx(bl[2].coeffs, start_y + 16);
-		video_copy_y_block_mmx(bl[3].coeffs, start_y + 24);
-		video_copy_ntsc_c_block_mmx(bl[4].coeffs,
+		_dv_video_copy_y_block_mmx(bl[0].coeffs, start_y);
+		_dv_video_copy_y_block_mmx(bl[1].coeffs, start_y + 8);
+		_dv_video_copy_y_block_mmx(bl[2].coeffs, start_y + 16);
+		_dv_video_copy_y_block_mmx(bl[3].coeffs, start_y + 24);
+		_dv_video_copy_ntsc_c_block_mmx(bl[4].coeffs,
 					  img_cr + y * DV_WIDTH / 2 + x / 2);
-		video_copy_ntsc_c_block_mmx(bl[5].coeffs,
+		_dv_video_copy_ntsc_c_block_mmx(bl[5].coeffs,
 					  img_cb + y * DV_WIDTH / 2 + x / 2);
 	}
 
@@ -938,7 +1119,7 @@ void dv_enc_register_input_filter(dv_enc_input_filter_t filter)
 	p->filter_name = NULL;
 }
 
-int get_dv_enc_input_filters(dv_enc_input_filter_t ** filters_,
+int dv_enc_get_input_filters(dv_enc_input_filter_t ** filters_,
 			     int * count)
 {
 	dv_enc_input_filter_t * p = filters;
@@ -952,17 +1133,15 @@ int get_dv_enc_input_filters(dv_enc_input_filter_t ** filters_,
 
 /****** public encoder implementation ***********************************/
 /* By Dan Dennedy <dan@dennedy.org> */
-void ycb_fill_macroblock(dv_encoder_t *dv_enc, dv_macroblock_t *mb)
+void _dv_ycb_fill_macroblock(dv_encoder_t *dv_enc, dv_macroblock_t *mb)
 {
 	int y = mb->y;
 	int x = mb->x;
 	dv_block_t *bl = mb->b;
-	int b;
-	int need_dct_248_rows[6];
 
 
 #if !ARCH_X86
-	if (dv_enc->isPAL || mb->x == DV_WIDTH- 16) { /* PAL or rightmost NTSC block */
+	if (dv_enc->isPAL) { /* PAL */
 		int i,j;
 		for (j = 0; j < 8; j++) {
 			for (i = 0; i < 8; i++) {
@@ -985,6 +1164,57 @@ void ycb_fill_macroblock(dv_encoder_t *dv_enc, dv_macroblock_t *mb)
 					      + x / 2 + i]
 					+ dv_enc->img_cb[(y + 2*j + 1) * DV_WIDTH/2
 						+ x / 2 + i]) >> 1;
+
+				if (dv_enc->clamp_luma == TRUE) {
+					bl[0].coeffs[8 * i + j] = CLAMP(bl[0].coeffs[8 * i + j],16-128,235-128);
+					bl[1].coeffs[8 * i + j] = CLAMP(bl[1].coeffs[8 * i + j],16-128,235-128);
+					bl[2].coeffs[8 * i + j] = CLAMP(bl[2].coeffs[8 * i + j],16-128,235-128);
+					bl[3].coeffs[8 * i + j] = CLAMP(bl[3].coeffs[8 * i + j],16-128,235-128);
+				}
+				if (dv_enc->clamp_chroma == TRUE) {
+					bl[4].coeffs[8 * i + j] = CLAMP(bl[4].coeffs[8 * i + j],16-128,240-128);
+					bl[5].coeffs[8 * i + j] = CLAMP(bl[5].coeffs[8 * i + j],16-128,240-128);
+				}
+			}
+		}
+	} else if (x == DV_WIDTH- 16) { /* rightmost NTSC block */
+		int i,j;
+		for (j = 0; j < 8; j++) {
+			for (i = 0; i < 8; i++) {
+				bl[0].coeffs[8 * i + j] = 
+					dv_enc->img_y[(y + j) * DV_WIDTH +  x + i];
+				bl[1].coeffs[8 * i + j] = 
+					dv_enc->img_y[(y + j) * DV_WIDTH +  x + 8 + i];
+				bl[2].coeffs[8 * i + j] = 
+					dv_enc->img_y[(y + 8 + j) * DV_WIDTH + x + i];
+				bl[3].coeffs[8 * i + j] = 
+					dv_enc->img_y[(y + 8 + j) * DV_WIDTH 
+					     + x + 8 + i];
+			}
+
+			for (i = 0; i < 4; i++) {
+				bl[4].coeffs[8 * i + j] = 
+					(dv_enc->img_cr[(y + j) * DV_WIDTH/2
+					       + x / 2 + i*2]
+					 + dv_enc->img_cr[(y + j) * DV_WIDTH/2 
+						 + x / 2 + 1 + i*2]) >> 1;
+				bl[5].coeffs[8 * i + j] = 
+					(dv_enc->img_cb[(y + j) * DV_WIDTH/2
+					       + x / 2 + i*2]
+					 + dv_enc->img_cb[(y + j) * DV_WIDTH/2 
+						 + x / 2 + 1 + i*2]) >> 1;
+				bl[4].coeffs[8 * (i + 4) + j] = 
+					(dv_enc->img_cr[(y + j + 8) * DV_WIDTH/2
+					       + x / 2 + i*2]
+					 + dv_enc->img_cr[(y + j + 8) * DV_WIDTH/2 
+						 + x / 2 + 1 + i*2]) >> 1;
+				bl[5].coeffs[8 * (i + 4) + j] = 
+					(dv_enc->img_cb[(y + j + 8) * DV_WIDTH/2
+					       + x / 2 + i*2]
+					 + dv_enc->img_cb[(y + j + 8) * DV_WIDTH/2 
+						 + x / 2 + 1 + i*2]) >> 1;
+			}
+			for (i = 0; i < 8; i++) {
 				if (dv_enc->clamp_luma == TRUE) {
 					bl[0].coeffs[8 * i + j] = CLAMP(bl[0].coeffs[8 * i + j],16-128,235-128);
 					bl[1].coeffs[8 * i + j] = CLAMP(bl[1].coeffs[8 * i + j],16-128,235-128);
@@ -1020,6 +1250,16 @@ void ycb_fill_macroblock(dv_encoder_t *dv_enc, dv_macroblock_t *mb)
 					 + dv_enc->img_cb[(y + j) * DV_WIDTH/2 
 						 + x / 2 + 1 + i*2]) >> 1;
 
+				if (dv_enc->clamp_luma == TRUE) {
+					bl[0].coeffs[8 * i + j] = CLAMP(bl[0].coeffs[8 * i + j],16-128,235-128);
+					bl[1].coeffs[8 * i + j] = CLAMP(bl[1].coeffs[8 * i + j],16-128,235-128);
+					bl[2].coeffs[8 * i + j] = CLAMP(bl[2].coeffs[8 * i + j],16-128,235-128);
+					bl[3].coeffs[8 * i + j] = CLAMP(bl[3].coeffs[8 * i + j],16-128,235-128);
+				}
+				if (dv_enc->clamp_chroma == TRUE) {
+					bl[4].coeffs[8 * i + j] = CLAMP(bl[4].coeffs[8 * i + j],16-128,240-128);
+					bl[5].coeffs[8 * i + j] = CLAMP(bl[5].coeffs[8 * i + j],16-128,240-128);
+				}
 			}
 		}
 	}
@@ -1036,25 +1276,61 @@ void ycb_fill_macroblock(dv_encoder_t *dv_enc, dv_macroblock_t *mb)
 		}
 	}
 #else
-	if (dv_enc->isPAL || mb->x == DV_WIDTH- 16) { /* PAL or rightmost NTSC block */
+	int b;
+	int need_dct_248_rows[6];
+
+	if (dv_enc->isPAL) { /* PAL or rightmost NTSC block */
 		short* start_y = dv_enc->img_y + y * DV_WIDTH + x;
-		ppm_copy_y_block_mmx(bl[0].coeffs, start_y);
-		ppm_copy_y_block_mmx(bl[1].coeffs, start_y + 8);
-		ppm_copy_y_block_mmx(bl[2].coeffs, start_y + 8 * DV_WIDTH);
-		ppm_copy_y_block_mmx(bl[3].coeffs, start_y + 8 * DV_WIDTH + 8);
-		ppm_copy_pal_c_block_mmx(bl[4].coeffs,
+		_dv_ppm_copy_y_block_mmx(bl[0].coeffs, start_y);
+		_dv_ppm_copy_y_block_mmx(bl[1].coeffs, start_y + 8);
+		_dv_ppm_copy_y_block_mmx(bl[2].coeffs, start_y + 8 * DV_WIDTH);
+		_dv_ppm_copy_y_block_mmx(bl[3].coeffs, start_y + 8 * DV_WIDTH + 8);
+		_dv_ppm_copy_pal_c_block_mmx(bl[4].coeffs,
 					 dv_enc->img_cr+y * DV_WIDTH/2+ x/2);
-		ppm_copy_pal_c_block_mmx(bl[5].coeffs,
+		_dv_ppm_copy_pal_c_block_mmx(bl[5].coeffs,
 					 dv_enc->img_cb+y * DV_WIDTH/2+ x/2);
+	} else if (x == DV_WIDTH- 16) { /* rightmost NTSC block */
+		short* start_y = dv_enc->img_y + y * DV_WIDTH + x;
+		int i,j;
+
+		_dv_ppm_copy_y_block_mmx(bl[0].coeffs, start_y);
+		_dv_ppm_copy_y_block_mmx(bl[1].coeffs, start_y + 8);
+		_dv_ppm_copy_y_block_mmx(bl[2].coeffs, start_y + 8 * DV_WIDTH);
+		_dv_ppm_copy_y_block_mmx(bl[3].coeffs, start_y + 8 * DV_WIDTH + 8);
+
+		for (j = 0; j < 8; j++) {
+			for (i = 0; i < 4; i++) {
+				bl[4].coeffs[8 * j + i] = 
+					(dv_enc->img_cr[(y + j) * DV_WIDTH/2
+					       + x / 2 + i*2]
+					 + dv_enc->img_cr[(y + j) * DV_WIDTH/2 
+						 + x / 2 + 1 + i*2]) >> 1;
+				bl[5].coeffs[8 * j + i] = 
+					(dv_enc->img_cb[(y + j) * DV_WIDTH/2
+					       + x / 2 + i*2]
+					 + dv_enc->img_cb[(y + j) * DV_WIDTH/2 
+						 + x / 2 + 1 + i*2]) >> 1;
+				bl[4].coeffs[8 * j + i + 4] = 
+					(dv_enc->img_cr[(y + j + 8) * DV_WIDTH/2
+					       + x / 2 + i*2]
+					 + dv_enc->img_cr[(y + j + 8) * DV_WIDTH/2 
+						 + x / 2 + 1 + i*2]) >> 1;
+				bl[5].coeffs[8 * j + i + 4] = 
+					(dv_enc->img_cb[(y + j + 8) * DV_WIDTH/2
+					       + x / 2 + i*2]
+					 + dv_enc->img_cb[(y + j + 8) * DV_WIDTH/2 
+						 + x / 2 + 1 + i*2]) >> 1;
+			}
+		}
 	} else {                              /* NTSC */
 		short* start_y = dv_enc->img_y + y * DV_WIDTH + x;
-		ppm_copy_y_block_mmx(bl[0].coeffs, start_y);
-		ppm_copy_y_block_mmx(bl[1].coeffs, start_y + 8);
-		ppm_copy_y_block_mmx(bl[2].coeffs, start_y + 16);
-		ppm_copy_y_block_mmx(bl[3].coeffs, start_y + 24);
-		ppm_copy_ntsc_c_block_mmx(bl[4].coeffs,
+		_dv_ppm_copy_y_block_mmx(bl[0].coeffs, start_y);
+		_dv_ppm_copy_y_block_mmx(bl[1].coeffs, start_y + 8);
+		_dv_ppm_copy_y_block_mmx(bl[2].coeffs, start_y + 16);
+		_dv_ppm_copy_y_block_mmx(bl[3].coeffs, start_y + 24);
+		_dv_ppm_copy_ntsc_c_block_mmx(bl[4].coeffs,
 					  dv_enc->img_cr + y*DV_WIDTH/2 + x/2);
-		ppm_copy_ntsc_c_block_mmx(bl[5].coeffs,
+		_dv_ppm_copy_ntsc_c_block_mmx(bl[5].coeffs,
 					  dv_enc->img_cb + y*DV_WIDTH/2 + x/2);
 	}
 
@@ -1067,21 +1343,21 @@ void ycb_fill_macroblock(dv_encoder_t *dv_enc, dv_macroblock_t *mb)
 	} else {
 		for (b = 0; b < 6; b++) {
 			need_dct_248_rows[b]
-				= need_dct_248_mmx_rows(bl[b].coeffs) + 1;
+				= _dv_need_dct_248_mmx_rows(bl[b].coeffs) + 1;
 		}
 	}
-	transpose_mmx(bl[0].coeffs);
-	transpose_mmx(bl[1].coeffs);
-	transpose_mmx(bl[2].coeffs);
-	transpose_mmx(bl[3].coeffs);
-	transpose_mmx(bl[4].coeffs);
-	transpose_mmx(bl[5].coeffs);
+	_dv_transpose_mmx(bl[0].coeffs);
+	_dv_transpose_mmx(bl[1].coeffs);
+	_dv_transpose_mmx(bl[2].coeffs);
+	_dv_transpose_mmx(bl[3].coeffs);
+	_dv_transpose_mmx(bl[4].coeffs);
+	_dv_transpose_mmx(bl[5].coeffs);
 
 	if (dv_enc->force_dct == -1) {
 		for (b = 0; b < 6; b++) {
 			bl[b].dct_mode = 
 				((need_dct_248_rows[b] * 65536 / 
-				  (need_dct_248_mmx_rows(bl[b].coeffs) + 1))
+				  (_dv_need_dct_248_mmx_rows(bl[b].coeffs) + 1))
 				 > DCT_248_THRESHOLD) ? DV_DCT_248 : DV_DCT_88;
 		}
 	}
